@@ -40,7 +40,7 @@
 #include <rak/file_stat.h>
 #include <rak/path.h>
 #include <rak/string_manip.h>
-#include <sigc++/bind.h>
+#include <sigc++/adaptors/bind.h>
 #include <torrent/rate.h>
 #include <torrent/hash_string.h>
 
@@ -79,7 +79,7 @@ apply_on_state_change(core::DownloadList::slot_map* slotMap, const torrent::Obje
 }
 
 torrent::Object
-apply_stop_on_ratio(const torrent::Object& rawArgs) {
+apply_on_ratio(int action, const torrent::Object& rawArgs) {
   const torrent::Object::list_type& args = rawArgs.as_list();
 
   if (args.empty())
@@ -95,20 +95,32 @@ apply_stop_on_ratio(const torrent::Object& rawArgs) {
   int64_t maxRatio  = argItr != args.end() ? rpc::convert_to_value(*argItr++) : 0;
 
   core::DownloadList* downloadList = control->core()->download_list();
-  core::Manager::DListItr itr = downloadList->begin();
 
-  while ((itr = std::find_if(itr, downloadList->end(), std::mem_fun(&core::Download::is_seeding)))
-         != downloadList->end()) {
+  for  (core::Manager::DListItr itr = downloadList->begin();
+        (itr = std::find_if(itr, downloadList->end(), std::mem_fun(&core::Download::is_seeding))) != downloadList->end();
+        itr++) {
     int64_t totalDone   = (*itr)->download()->bytes_done();
     int64_t totalUpload = (*itr)->download()->up_rate()->total();
 
-    if ((totalUpload >= minUpload && totalUpload * 100 >= totalDone * minRatio) ||
-        (maxRatio > 0 && totalUpload * 100 > totalDone * maxRatio)) {
-      downloadList->stop_try(*itr);
-      rpc::call_command_d("d.set_ignore_commands", *itr, (int64_t)1);
+    if (!(totalUpload >= minUpload && totalUpload * 100 >= totalDone * minRatio) &&
+        !(maxRatio > 0 && totalUpload * 100 > totalDone * maxRatio))
+      continue;
+
+    bool success;
+
+    switch (action) {
+    case core::DownloadList::SLOTS_CLOSE: success = downloadList->close_try(*itr); break;
+    case core::DownloadList::SLOTS_STOP:  success = downloadList->stop_try(*itr); break;
+    default: success = false; break;
     }
 
-    ++itr;
+    if (!success)
+      continue;
+
+    rpc::call_command("d.set_ignore_commands", (int64_t)1, rpc::make_target(*itr));
+
+    for (torrent::Object::list_type::const_iterator itr2 = argItr; itr2 != args.end(); itr2++)
+      rpc::parse_command_object(rpc::make_target(*itr), *itr2);
   }
 
   return torrent::Object();
@@ -117,11 +129,11 @@ apply_stop_on_ratio(const torrent::Object& rawArgs) {
 torrent::Object
 apply_start_tied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
-    if (rpc::call_command_d_value("d.get_state", *itr) == 1)
+    if (rpc::call_command_value("d.get_state", rpc::make_target(*itr)) == 1)
       continue;
 
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_d_string("d.get_tied_to_file", *itr);
+    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && fs.update(rak::path_expand(tiedToFile)))
       control->core()->download_list()->start_try(*itr);
@@ -133,11 +145,11 @@ apply_start_tied() {
 torrent::Object
 apply_stop_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
-    if (rpc::call_command_d_value("d.get_state", *itr) == 0)
+    if (rpc::call_command_value("d.get_state", rpc::make_target(*itr)) == 0)
       continue;
 
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_d_string("d.get_tied_to_file", *itr);
+    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
       control->core()->download_list()->stop_try(*itr);
@@ -150,9 +162,9 @@ torrent::Object
 apply_close_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ++itr) {
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_d_string("d.get_tied_to_file", *itr);
+    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
 
-    if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)) && control->core()->download_list()->stop_try(*itr))
+    if (rpc::call_command_value("d.get_ignore_commands", rpc::make_target(*itr)) == 0 && !tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile)))
       control->core()->download_list()->close(*itr);
   }
 
@@ -163,11 +175,11 @@ torrent::Object
 apply_remove_untied() {
   for (core::DownloadList::iterator itr = control->core()->download_list()->begin(); itr != control->core()->download_list()->end(); ) {
     rak::file_stat fs;
-    const std::string& tiedToFile = rpc::call_command_d_string("d.get_tied_to_file", *itr);
+    const std::string& tiedToFile = rpc::call_command_string("d.get_tied_to_file", rpc::make_target(*itr));
 
     if (!tiedToFile.empty() && !fs.update(rak::path_expand(tiedToFile))) {
       // Need to clear tied_to_file so it doesn't try to delete it.
-      rpc::call_command_d("d.set_tied_to_file", *itr, std::string());
+      rpc::call_command("d.set_tied_to_file", std::string(), rpc::make_target(*itr));
 
       itr = control->core()->download_list()->erase(itr);
 
@@ -224,6 +236,7 @@ void
 apply_close_low_diskspace(int64_t arg) {
   core::DownloadList* downloadList = control->core()->download_list();
 
+  bool closed = false;
   core::Manager::DListItr itr = downloadList->begin();
 
   while ((itr = std::find_if(itr, downloadList->end(), std::mem_fun(&core::Download::is_downloading)))
@@ -233,10 +246,15 @@ apply_close_low_diskspace(int64_t arg) {
 
       (*itr)->set_hash_failed(true);
       (*itr)->set_message(std::string("Low diskspace."));
+
+      closed = true;
     }
 
     ++itr;
   }
+
+  if (closed)
+    control->core()->push_log("Closed torrents due to low diskspace.");    
 }
 
 torrent::Object
@@ -268,7 +286,7 @@ apply_download_list(const torrent::Object& rawArgs) {
 }
 
 torrent::Object
-apply_call_download(const torrent::Object& rawArgs) {
+d_multicall(const torrent::Object& rawArgs) {
   const torrent::Object::list_type&          args = rawArgs.as_list();
 
   if (args.empty())
@@ -306,7 +324,7 @@ apply_call_download(const torrent::Object& rawArgs) {
 
     for (torrent::Object::list_type::const_iterator cItr = ++args.begin(), cLast = args.end(); cItr != args.end(); cItr++) {
       const std::string& cmd = cItr->as_string();
-      row.push_back(rpc::parse_command_d_single(*vItr, cmd.c_str(), cmd.c_str() + cmd.size()));
+      row.push_back(rpc::parse_command(rpc::make_target(*vItr), cmd.c_str(), cmd.c_str() + cmd.size()).first);
     }
   }
 
@@ -322,23 +340,24 @@ initialize_command_events() {
   ADD_VARIABLE_BOOL("session_lock", true);
   ADD_VARIABLE_BOOL("session_on_completion", true);
 
-  ADD_COMMAND_SLOT_PRIVATE("on_insert",       call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_insert()));
-  ADD_COMMAND_SLOT_PRIVATE("on_erase",        call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_erase()));
-  ADD_COMMAND_SLOT_PRIVATE("on_open",         call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_open()));
-  ADD_COMMAND_SLOT_PRIVATE("on_close",        call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_close()));
-  ADD_COMMAND_SLOT_PRIVATE("on_start",        call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_start()));
-  ADD_COMMAND_SLOT_PRIVATE("on_stop",         call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_stop()));
-  ADD_COMMAND_SLOT_PRIVATE("on_hash_queued",  call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_queued()));
-  ADD_COMMAND_SLOT_PRIVATE("on_hash_removed", call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_removed()));
-  ADD_COMMAND_SLOT_PRIVATE("on_hash_done",    call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_done()));
-  ADD_COMMAND_SLOT_PRIVATE("on_finished",     call_list, rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_finished()));
+  ADD_COMMAND_LIST("on_insert",       rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_insert()));
+  ADD_COMMAND_LIST("on_erase",        rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_erase()));
+  ADD_COMMAND_LIST("on_open",         rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_open()));
+  ADD_COMMAND_LIST("on_close",        rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_close()));
+  ADD_COMMAND_LIST("on_start",        rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_start()));
+  ADD_COMMAND_LIST("on_stop",         rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_stop()));
+  ADD_COMMAND_LIST("on_hash_queued",  rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_queued()));
+  ADD_COMMAND_LIST("on_hash_removed", rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_removed()));
+  ADD_COMMAND_LIST("on_hash_done",    rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_hash_done()));
+  ADD_COMMAND_LIST("on_finished",     rak::bind_ptr_fn(&apply_on_state_change, &downloadList->slot_map_finished()));
 
-  ADD_COMMAND_SLOT_PRIVATE("stop_on_ratio",   call_list, rak::ptr_fn(&apply_stop_on_ratio));
+  ADD_COMMAND_LIST("stop_on_ratio",   rak::bind_ptr_fn(&apply_on_ratio, (int)core::DownloadList::SLOTS_STOP));
+  ADD_COMMAND_LIST("close_on_ratio",  rak::bind_ptr_fn(&apply_on_ratio, (int)core::DownloadList::SLOTS_CLOSE));
 
-  ADD_COMMAND_SLOT_PRIVATE("start_tied",      call_string, rpc::object_fn(&apply_start_tied));
-  ADD_COMMAND_SLOT_PRIVATE("stop_untied",     call_string, rpc::object_fn(&apply_stop_untied));
-  ADD_COMMAND_SLOT_PRIVATE("close_untied",    call_string, rpc::object_fn(&apply_close_untied));
-  ADD_COMMAND_SLOT_PRIVATE("remove_untied",   call_string, rpc::object_fn(&apply_remove_untied));
+  ADD_COMMAND_VOID("start_tied",      &apply_start_tied);
+  ADD_COMMAND_VOID("stop_untied",     &apply_stop_untied);
+  ADD_COMMAND_VOID("close_untied",    &apply_close_untied);
+  ADD_COMMAND_VOID("remove_untied",   &apply_remove_untied);
 
   ADD_COMMAND_LIST("schedule",                rak::ptr_fn(&apply_schedule));
   ADD_COMMAND_STRING_UN("schedule_remove",    rak::make_mem_fun(control->command_scheduler(), &rpc::CommandScheduler::erase_str));
@@ -349,10 +368,14 @@ initialize_command_events() {
   ADD_COMMAND_LIST("load",                    rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_tied));
   ADD_COMMAND_LIST("load_verbose",            rak::bind_ptr_fn(&apply_load, core::Manager::create_tied));
   ADD_COMMAND_LIST("load_start",              rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_tied | core::Manager::create_start));
-  ADD_COMMAND_LIST("load_start_verbose",      rak::bind_ptr_fn(&apply_load, core::Manager::create_tied | core::Manager::create_start));
+  ADD_COMMAND_LIST("load_start_verbose",      rak::bind_ptr_fn(&apply_load, core::Manager::create_tied  | core::Manager::create_start));
+  ADD_COMMAND_LIST("load_raw",                rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_raw_data));
+  ADD_COMMAND_LIST("load_raw_verbose",        rak::bind_ptr_fn(&apply_load, core::Manager::create_raw_data));
+  ADD_COMMAND_LIST("load_raw_start",          rak::bind_ptr_fn(&apply_load, core::Manager::create_quiet | core::Manager::create_start | core::Manager::create_raw_data));
 
   ADD_COMMAND_VALUE_UN("close_low_diskspace", std::ptr_fun(&apply_close_low_diskspace));
 
   ADD_COMMAND_LIST("download_list",           rak::ptr_fn(&apply_download_list));
-  ADD_COMMAND_LIST("call_download",           rak::ptr_fn(&apply_call_download));
+  ADD_COMMAND_LIST("d.multicall",             rak::ptr_fn(&d_multicall));
+  ADD_COMMAND_COPY("call_download",           call_list, "i:", "");
 }
