@@ -37,40 +37,42 @@
 #ifndef RTORRENT_UTILS_COMMAND_HELPERS_H
 #define RTORRENT_UTILS_COMMAND_HELPERS_H
 
+#include "rpc/command_slot.h"
 #include "rpc/parse_commands.h"
 
 namespace rpc {
-  class CommandSlot;
   class CommandVariable;
-  class CommandDownloadSlot;
-  class CommandFileSlot;
-  class CommandPeerSlot;
-  class CommandTrackerSlot;
 }
 
 // By using a static array we avoid allocating the variables on the
 // heap. This should reduce memory use and improve cache locality.
-#define COMMAND_SLOTS_SIZE          100
+#define COMMAND_SLOTS_SIZE          150
 #define COMMAND_VARIABLES_SIZE      100
-#define COMMAND_DOWNLOAD_SLOTS_SIZE 100
+#define COMMAND_DOWNLOAD_SLOTS_SIZE 150
 #define COMMAND_FILE_SLOTS_SIZE     20
+#define COMMAND_FILE_ITR_SLOTS_SIZE 10
 #define COMMAND_PEER_SLOTS_SIZE     20
 #define COMMAND_TRACKER_SLOTS_SIZE  15
+#define COMMAND_ANY_SLOTS_SIZE      20
 
 #define ADDING_COMMANDS
 
-extern rpc::CommandSlot          commandSlots[COMMAND_SLOTS_SIZE];
-extern rpc::CommandSlot*         commandSlotsItr;
+extern rpc::CommandSlot<void>    commandSlots[COMMAND_SLOTS_SIZE];
+extern rpc::CommandSlot<void>*   commandSlotsItr;
 extern rpc::CommandVariable      commandVariables[COMMAND_VARIABLES_SIZE];
 extern rpc::CommandVariable*     commandVariablesItr;
-extern rpc::CommandDownloadSlot  commandDownloadSlots[COMMAND_DOWNLOAD_SLOTS_SIZE];
-extern rpc::CommandDownloadSlot* commandDownloadSlotsItr;
-extern rpc::CommandFileSlot      commandFileSlots[COMMAND_FILE_SLOTS_SIZE];
-extern rpc::CommandFileSlot*     commandFileSlotsItr;
-extern rpc::CommandPeerSlot      commandPeerSlots[COMMAND_PEER_SLOTS_SIZE];
-extern rpc::CommandPeerSlot*     commandPeerSlotsItr;
-extern rpc::CommandTrackerSlot   commandTrackerSlots[COMMAND_TRACKER_SLOTS_SIZE];
-extern rpc::CommandTrackerSlot*  commandTrackerSlotsItr;
+extern rpc::CommandSlot<core::Download*>             commandDownloadSlots[COMMAND_DOWNLOAD_SLOTS_SIZE];
+extern rpc::CommandSlot<core::Download*>*            commandDownloadSlotsItr;
+extern rpc::CommandSlot<torrent::File*>              commandFileSlots[COMMAND_FILE_SLOTS_SIZE];
+extern rpc::CommandSlot<torrent::File*>*             commandFileSlotsItr;
+extern rpc::CommandSlot<torrent::FileListIterator*>  commandFileItrSlots[COMMAND_FILE_ITR_SLOTS_SIZE];
+extern rpc::CommandSlot<torrent::FileListIterator*>* commandFileItrSlotsItr;
+extern rpc::CommandSlot<torrent::Peer*>              commandPeerSlots[COMMAND_PEER_SLOTS_SIZE];
+extern rpc::CommandSlot<torrent::Peer*>*             commandPeerSlotsItr;
+extern rpc::CommandSlot<torrent::Tracker*>           commandTrackerSlots[COMMAND_TRACKER_SLOTS_SIZE];
+extern rpc::CommandSlot<torrent::Tracker*>*          commandTrackerSlotsItr;
+extern rpc::CommandSlot<rpc::target_type>            commandAnySlots[COMMAND_ANY_SLOTS_SIZE];
+extern rpc::CommandSlot<rpc::target_type>*           commandAnySlotsItr;
 
 void initialize_commands();
 
@@ -88,22 +90,26 @@ add_variable("get_" key, "set_" key, key, &rpc::CommandVariable::get_value, &rpc
 #define ADD_VARIABLE_STRING(key, defaultValue) \
 add_variable("get_" key, "set_" key, key, &rpc::CommandVariable::get_string, &rpc::CommandVariable::set_string, std::string(defaultValue));
 
-#define ADD_VARIABLE_C_STRING(key, defaultValue) \
-add_variable("get_" key, NULL, NULL, &rpc::CommandVariable::get_string, NULL, std::string(defaultValue));
+#define ADD_C_STRING(key, defaultValue) \
+add_variable(key, NULL, NULL, &rpc::CommandVariable::get_string, NULL, std::string(defaultValue));
 
 #define ADD_COMMAND_SLOT(key, function, slot, parm, doc)    \
   commandSlotsItr->set_slot(slot); \
-  rpc::commands.insert_generic(key, commandSlotsItr++, &rpc::CommandSlot::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
+  rpc::commands.insert_generic(key, commandSlotsItr++, &rpc::CommandSlot<void>::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
+
+#define ADD_ANY_SLOT(key, function, slot, parm, doc)    \
+  commandAnySlotsItr->set_slot(slot); \
+  rpc::commands.insert_any(key, commandAnySlotsItr++, &rpc::CommandSlot<rpc::target_type>::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
 
 #define ADD_COMMAND_SLOT_PRIVATE(key, function, slot) \
   commandSlotsItr->set_slot(slot); \
-  rpc::commands.insert_generic(key, commandSlotsItr++, &rpc::CommandSlot::function, rpc::CommandMap::flag_dont_delete, NULL, NULL);
+  rpc::commands.insert_generic(key, commandSlotsItr++, &rpc::CommandSlot<void>::function, rpc::CommandMap::flag_dont_delete, NULL, NULL);
 
 #define ADD_COMMAND_COPY(key, function, parm, doc) \
-  rpc::commands.insert_generic(key, (commandSlotsItr - 1), &rpc::CommandSlot::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
+  rpc::commands.insert_generic(key, (commandSlotsItr - 1), &rpc::CommandSlot<void>::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
 
 #define ADD_COMMAND_COPY_PRIVATE(key, function) \
-  rpc::commands.insert_generic(key, (commandSlotsItr - 1), &rpc::CommandSlot::function, rpc::CommandMap::flag_dont_delete, NULL, NULL);
+  rpc::commands.insert_generic(key, (commandSlotsItr - 1), &rpc::CommandSlot<void>::function, rpc::CommandMap::flag_dont_delete, NULL, NULL);
 
 #define ADD_COMMAND_VALUE_TRI(key, set, get) \
   ADD_COMMAND_SLOT_PRIVATE(key, call_value, rpc::object_value_fn(set))      \
@@ -115,15 +121,22 @@ add_variable("get_" key, NULL, NULL, &rpc::CommandVariable::get_string, NULL, st
   ADD_COMMAND_COPY("set_" key,  call_value, "i:i", "")                      \
   ADD_COMMAND_SLOT("get_" key,  call_unknown, rpc::object_void_fn(get), "i:", "")
 
-#define ADD_COMMAND_VALUE_TRI_OCT(key, set, get) \
+#define ADD_COMMAND_VALUE_SET_OCT(prefix, key, set)                 \
   ADD_COMMAND_SLOT_PRIVATE(key, call_value_oct, rpc::object_value_fn(set)) \
-  ADD_COMMAND_COPY("set_" key,  call_value, "i:i", "")                      \
-  ADD_COMMAND_SLOT("get_" key,  call_unknown, rpc::object_void_fn(get), "i:", "")
+  ADD_COMMAND_COPY(prefix "set_" key,  call_value, "i:i", "")
 
-#define ADD_COMMAND_STRING_TRI(key, set, get) \
+#define ADD_COMMAND_VALUE_TRI_OCT(prefix, key, set, get)                 \
+  ADD_COMMAND_SLOT_PRIVATE(key, call_value_oct, rpc::object_value_fn(set)) \
+  ADD_COMMAND_COPY(prefix "set_" key,  call_value, "i:i", "")                      \
+  ADD_COMMAND_SLOT(prefix "get_" key,  call_unknown, rpc::object_void_fn(get), "i:", "")
+
+#define ADD_COMMAND_STRING_PREFIX(prefix, key, set, get) \
   ADD_COMMAND_SLOT_PRIVATE(key, call_string, rpc::object_string_fn(set))      \
-  ADD_COMMAND_COPY("set_" key,  call_string, "i:s", "") \
-  ADD_COMMAND_SLOT("get_" key,  call_unknown, rpc::object_void_fn(get), "s:", "")
+  ADD_COMMAND_COPY(prefix "set_" key,  call_string, "i:s", "") \
+  ADD_COMMAND_SLOT(prefix "get_" key,  call_unknown, rpc::object_void_fn(get), "s:", "")
+
+#define ADD_COMMAND_STRING_TRI(key, set, get)                    \
+  ADD_COMMAND_STRING_PREFIX("", key, set, get)
 
 #define ADD_COMMAND_VOID(key, slot) \
   ADD_COMMAND_SLOT(key, call_unknown, rpc::object_void_fn(slot), "i:", "")
@@ -131,6 +144,9 @@ add_variable("get_" key, NULL, NULL, &rpc::CommandVariable::get_string, NULL, st
 #define ADD_COMMAND_VALUE(key, slot) \
   ADD_COMMAND_SLOT(key, call_value, slot, "i:i", "")
 
+#define ADD_ANY_VALUE(key, slot) \
+  ADD_ANY_SLOT(key, call_value, slot, "i:i", "")
+
 #define ADD_COMMAND_VALUE_UN(key, slot) \
   ADD_COMMAND_SLOT(key, call_value, rpc::object_value_fn(slot), "i:i", "")
 
@@ -143,6 +159,12 @@ add_variable("get_" key, NULL, NULL, &rpc::CommandVariable::get_string, NULL, st
 #define ADD_COMMAND_LIST(key, slot) \
   ADD_COMMAND_SLOT(key, call_list, slot, "i:", "")
 
+#define ADD_COMMAND_NONE(key, slot) \
+  ADD_COMMAND_SLOT(key, call_unknown, slot, "i:", "")
+
+#define ADD_ANY_NONE(key, slot) \
+  ADD_ANY_SLOT(key, call_unknown, slot, "i:", "")
+
 #define ADD_COMMAND_NONE_L(key, slot) \
   ADD_COMMAND_SLOT(key, call_unknown, slot, "A:", "")
 
