@@ -40,8 +40,6 @@
 #include <torrent/tracker.h>
 
 #include "core/manager.h"
-#include "rpc/command_slot.h"
-#include "rpc/command_tracker_slot.h"
 
 #include "globals.h"
 #include "control.h"
@@ -57,41 +55,47 @@ apply_t_set_enabled(torrent::Tracker* tracker, int64_t state) {
 
 #define ADD_CT_SLOT(key, function, slot, parm, doc)    \
   commandTrackerSlotsItr->set_slot(slot); \
-  rpc::commands.insert_tracker(key, commandTrackerSlotsItr++, &rpc::CommandTrackerSlot::function, rpc::CommandMap::flag_dont_delete, parm, doc);
+  rpc::commands.insert_tracker(key, commandTrackerSlotsItr++, &rpc::CommandSlot<torrent::Tracker*>::function, rpc::CommandMap::flag_dont_delete, parm, doc);
 
 #define ADD_CT_SLOT_PUBLIC(key, function, slot, parm, doc)    \
   commandTrackerSlotsItr->set_slot(slot); \
-  rpc::commands.insert_tracker(key, commandTrackerSlotsItr++, &rpc::CommandTrackerSlot::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
+  rpc::commands.insert_tracker(key, commandTrackerSlotsItr++, &rpc::CommandSlot<torrent::Tracker*>::function, rpc::CommandMap::flag_dont_delete | rpc::CommandMap::flag_public_xmlrpc, parm, doc);
 
 #define ADD_CT_VOID(key, slot) \
-  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_t_fn(slot), "i:", "")
+  ADD_CT_SLOT_PUBLIC("t." key, call_unknown,     rpc::object_void_fn<torrent::Tracker*>(slot), "i:", "")
+
+#define ADD_CT_VOID_UNI(key, get) \
+  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_fn<torrent::Tracker*>(get), "i:", "")
 
 #define ADD_CT_VALUE_UNI(key, get) \
-  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_t_fn(get), "i:", "")
+  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_fn<torrent::Tracker*>(get), "i:", "")
 
 #define ADD_CT_VALUE_BI(key, set, get) \
-  ADD_CT_SLOT_PUBLIC("t.set_" key, call_value, rpc::object_value_t_fn(set), "i:i", "") \
-  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_t_fn(get), "i:", "")
+  ADD_CT_SLOT_PUBLIC("t.set_" key, call_value,   rpc::object_value_fn<torrent::Tracker*>(set), "i:i", "") \
+  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_fn<torrent::Tracker*>(get), "i:", "")
+
+#define ADD_CT_BOOL(key, set, get) \
+  ADD_CT_SLOT_PUBLIC("t.set_" key, call_value,   rpc::object_value_fn<torrent::Tracker*>(set), "i:i", "") \
+  ADD_CT_SLOT_PUBLIC("t.is_" key, call_unknown,  rpc::object_void_fn<torrent::Tracker*>(get), "i:", "")
 
 #define ADD_CT_STRING_UNI(key, get) \
-  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_t_fn(get), "s:", "")
+  ADD_CT_SLOT_PUBLIC("t.get_" key, call_unknown, rpc::object_void_fn<torrent::Tracker*>(get), "s:", "")
 
 void
 initialize_command_tracker() {
-  ADD_CT_STRING_UNI("url",              std::mem_fun(&torrent::Tracker::url));
-  ADD_CT_VALUE_UNI("group",             std::mem_fun(&torrent::Tracker::group));
-  ADD_CT_VALUE_UNI("type",              std::mem_fun(&torrent::Tracker::tracker_type));
-  ADD_CT_STRING_UNI("id",               std::mem_fun(&torrent::Tracker::tracker_id));
+  ADD_CT_STRING_UNI("url",             std::mem_fun(&torrent::Tracker::url));
+  ADD_CT_VOID_UNI("group",             std::mem_fun(&torrent::Tracker::group));
+  ADD_CT_VOID_UNI("type",              std::mem_fun(&torrent::Tracker::tracker_type));
+  ADD_CT_STRING_UNI("id",              std::mem_fun(&torrent::Tracker::tracker_id));
 
-  ADD_CT_VALUE_BI("enabled",            std::ptr_fun(&apply_t_set_enabled), std::mem_fun(&torrent::Tracker::is_enabled));
+  ADD_CT_VOID("is_open",               std::mem_fun(&torrent::Tracker::is_open));
+  ADD_CT_BOOL("enabled",               std::ptr_fun(&apply_t_set_enabled), std::mem_fun(&torrent::Tracker::is_enabled));
+  
+  ADD_CT_VOID_UNI("normal_interval",   std::mem_fun(&torrent::Tracker::normal_interval));
+  ADD_CT_VOID_UNI("min_interval",      std::mem_fun(&torrent::Tracker::min_interval));
 
-  ADD_CT_VALUE_UNI("is_open",           std::mem_fun(&torrent::Tracker::is_open));
-
-  ADD_CT_VALUE_UNI("normal_interval",   std::mem_fun(&torrent::Tracker::normal_interval));
-  ADD_CT_VALUE_UNI("min_interval",      std::mem_fun(&torrent::Tracker::min_interval));
-
-  ADD_CT_VALUE_UNI("scrape_time_last",  std::mem_fun(&torrent::Tracker::scrape_time_last));
-  ADD_CT_VALUE_UNI("scrape_complete",   std::mem_fun(&torrent::Tracker::scrape_complete));
-  ADD_CT_VALUE_UNI("scrape_incomplete", std::mem_fun(&torrent::Tracker::scrape_incomplete));
-  ADD_CT_VALUE_UNI("scrape_downloaded", std::mem_fun(&torrent::Tracker::scrape_downloaded));
+  ADD_CT_VOID_UNI("scrape_time_last",  std::mem_fun(&torrent::Tracker::scrape_time_last));
+  ADD_CT_VOID_UNI("scrape_complete",   std::mem_fun(&torrent::Tracker::scrape_complete));
+  ADD_CT_VOID_UNI("scrape_incomplete", std::mem_fun(&torrent::Tracker::scrape_incomplete));
+  ADD_CT_VOID_UNI("scrape_downloaded", std::mem_fun(&torrent::Tracker::scrape_downloaded));
 }
