@@ -36,8 +36,8 @@
 
 #include "config.h"
 
-#include <sigc++/bind.h>
-#include <sigc++/hide.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/adaptors/hide.h>
 #include <sigc++/signal.h>
 #include <rak/path.h>
 #include <rak/functional.h>
@@ -52,6 +52,7 @@
 
 #include "control.h"
 #include "download.h"
+#include "manager.h"
 
 namespace core {
 
@@ -61,7 +62,8 @@ Download::Download(download_type d) :
 
   m_hashFailed(false),
 
-  m_chunksFailed(0) {
+  m_chunksFailed(0),
+  m_resumeFlags(~uint32_t()) {
 
   m_connTrackerSucceded = m_download.signal_tracker_succeded(sigc::bind(sigc::mem_fun(*this, &Download::receive_tracker_msg), ""));
   m_connTrackerFailed   = m_download.signal_tracker_failed(sigc::mem_fun(*this, &Download::receive_tracker_msg));
@@ -157,6 +159,8 @@ void
 Download::set_root_directory(const std::string& path) {
   torrent::FileList* fileList = m_download.file_list();
 
+  control->core()->download_list()->close_directly(this);
+
   if (path.empty()) {
     fileList->set_root_dir("./" + (fileList->is_multi_file() ? m_download.name() : std::string()));
 
