@@ -61,13 +61,12 @@
 
 namespace core {
 
-DownloadFactory::DownloadFactory(const std::string& uri, Manager* m) :
+DownloadFactory::DownloadFactory(Manager* m) :
   m_manager(m),
   m_stream(NULL),
   m_commited(false),
   m_loaded(false),
 
-  m_uri(uri),
   m_session(false),
   m_start(false),
   m_printLog(true) {
@@ -90,10 +89,21 @@ DownloadFactory::~DownloadFactory() {
 }
 
 void
-DownloadFactory::load() {
+DownloadFactory::load(const std::string& uri) {
+  m_uri = uri;
   priority_queue_insert(&taskScheduler, &m_taskLoad, cachedTime);
 }
 
+// This function must be called before DownloadFactory::commit().
+void
+DownloadFactory::load_raw_data(const std::string& input) {
+  if (m_stream)
+    throw torrent::internal_error("DownloadFactory::load*() called on an object with m_stream != NULL");
+
+  m_stream = new std::stringstream(input);
+  m_loaded = true;
+}
+
 void
 DownloadFactory::commit() {
   priority_queue_insert(&taskScheduler, &m_taskCommit, cachedTime);
@@ -102,7 +112,7 @@ DownloadFactory::commit() {
 void
 DownloadFactory::receive_load() {
   if (m_stream)
-    throw torrent::internal_error("DownloadFactory::load() called on an object with m_stream != NULL");
+    throw torrent::internal_error("DownloadFactory::load*() called on an object with m_stream != NULL");
 
   if (std::strncmp(m_uri.c_str(), "http://", 7) == 0 ||
       std::strncmp(m_uri.c_str(), "https://", 8) == 0 ||
@@ -174,28 +184,34 @@ DownloadFactory::receive_success() {
 
   initialize_rtorrent(download, rtorrent);
 
+  if (!rtorrent->has_key_string("custom1")) rtorrent->insert_key("custom1", std::string());
+  if (!rtorrent->has_key_string("custom2")) rtorrent->insert_key("custom2", std::string());
+  if (!rtorrent->has_key_string("custom3")) rtorrent->insert_key("custom3", std::string());
+  if (!rtorrent->has_key_string("custom4")) rtorrent->insert_key("custom4", std::string());
+  if (!rtorrent->has_key_string("custom5")) rtorrent->insert_key("custom5", std::string());
+
   // Move to 'rtorrent'.
-  rpc::call_command_d("d.set_connection_leech", download, m_variables["connection_leech"]);
-  rpc::call_command_d("d.set_connection_seed", download,  m_variables["connection_seed"]);
+  rpc::call_command("d.set_connection_leech", m_variables["connection_leech"], rpc::make_target(download));
+  rpc::call_command("d.set_connection_seed",  m_variables["connection_seed"], rpc::make_target(download));
 
-  rpc::call_command_d("d.set_max_uploads", download,      rpc::call_command_void("get_max_uploads"));
-  rpc::call_command_d("d.set_min_peers", download,        rpc::call_command_void("get_min_peers"));
-  rpc::call_command_d("d.set_max_peers", download,        rpc::call_command_void("get_max_peers"));
-  rpc::call_command_d("d.set_tracker_numwant", download,  rpc::call_command_void("get_tracker_numwant"));
+  rpc::call_command("d.set_max_uploads",      rpc::call_command_void("get_max_uploads"), rpc::make_target(download));
+  rpc::call_command("d.set_min_peers",        rpc::call_command_void("get_min_peers"), rpc::make_target(download));
+  rpc::call_command("d.set_max_peers",        rpc::call_command_void("get_max_peers"), rpc::make_target(download));
+  rpc::call_command("d.set_tracker_numwant",  rpc::call_command_void("get_tracker_numwant"), rpc::make_target(download));
 
-  if (rpc::call_command_d_value("d.get_complete", download) != 0) {
+  if (rpc::call_command_value("d.get_complete", rpc::make_target(download)) != 0) {
     if (rpc::call_command_value("get_min_peers_seed") >= 0)
-      rpc::call_command_d("d.set_min_peers", download, rpc::call_command_void("get_min_peers_seed"));
+      rpc::call_command("d.set_min_peers", rpc::call_command_void("get_min_peers_seed"), rpc::make_target(download));
 
     if (rpc::call_command_value("get_max_peers_seed") >= 0)
-      rpc::call_command_d("d.set_max_peers", download, rpc::call_command_void("get_max_peers_seed"));
+      rpc::call_command("d.set_max_peers", rpc::call_command_void("get_max_peers_seed"), rpc::make_target(download));
   }
 
   if (!rpc::call_command_value("get_use_udp_trackers"))
     download->enable_udp_trackers(false);
 
   if (rpc::call_command_value("get_max_file_size") > 0)
-    rpc::call_command_d("d.set_max_file_size", download, rpc::call_command_void("get_max_file_size"));
+    rpc::call_command("d.set_max_file_size", rpc::call_command_void("get_max_file_size"), rpc::make_target(download));
 
   // Check first if we already have these values set in the session
   // torrent, so that it is safe to change the values.
@@ -207,12 +223,14 @@ DownloadFactory::receive_success() {
                             rpc::call_command_string("split_suffix"));
 
   if (!rtorrent->has_key_string("directory"))
-    rpc::call_command_d("d.set_directory", download, m_variables["directory"]);
+    rpc::call_command("d.set_directory", m_variables["directory"], rpc::make_target(download));
   else
-    rpc::call_command_d("d.set_directory", download, rtorrent->get_key("directory"));
+    rpc::call_command("d.set_directory", rtorrent->get_key("directory"), rpc::make_target(download));
 
   if (!m_session && m_variables["tied_to_file"].as_value())
-    rpc::call_command_d("d.set_tied_to_file", download, m_uri);
+    rpc::call_command("d.set_tied_to_file", m_uri, rpc::make_target(download));
+
+  rpc::call_command("d.set_peer_exchange", rpc::call_command_value("get_peer_exchange"), rpc::make_target(download));
 
   torrent::Object& resumeObject = root->has_key_map("libtorrent_resume")
     ? root->get_key("libtorrent_resume")
@@ -264,8 +282,8 @@ DownloadFactory::receive_success() {
   if (m_session) {
     // This torrent was queued for hashing or hashing when the session
     // file was saved. Or it was in a started state.
-    if (rpc::call_command_d_value("d.get_hashing", download) != Download::variable_hashing_stopped ||
-        rpc::call_command_d_value("d.get_state", download) != 0)
+    if (rpc::call_command_value("d.get_hashing", rpc::make_target(download)) != Download::variable_hashing_stopped ||
+        rpc::call_command_value("d.get_state", rpc::make_target(download)) != 0)
       m_manager->download_list()->resume(download);
 
   } else {
@@ -314,9 +332,9 @@ DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorre
     rtorrent->insert_key("tied_to_file", std::string());
 
   if (rtorrent->has_key_value("priority"))
-    rpc::call_command_d("d.set_priority", download, rtorrent->get_key_value("priority") % 4);
+    rpc::call_command("d.set_priority", rtorrent->get_key_value("priority") % 4, rpc::make_target(download));
   else
-    rpc::call_command_d("d.set_priority", download, (int64_t)2);
+    rpc::call_command("d.set_priority", (int64_t)2, rpc::make_target(download));
 
   if (rtorrent->has_key_value("key")) {
     download->tracker_list()->set_key(rtorrent->get_key_value("key"));
