@@ -41,7 +41,7 @@
 #include <list>
 #include <map>
 #include <string>
-#include <sigc++/slot.h>
+#include <sigc++/functors/slot.h>
 
 namespace torrent {
   class HashString;
@@ -101,6 +101,8 @@ public:
   void                open_throw(Download* d);
 
   void                close(Download* d);
+  bool                close_try(Download* d);
+  void                close_directly(Download* d);
   void                close_quick(Download* d);
   void                close_throw(Download* d);
 
@@ -110,8 +112,10 @@ public:
   void                stop_normal(Download* d);
   bool                stop_try(Download* d);
 
-  void                resume(Download* d);
-  void                pause(Download* d);
+  void                resume(Download* d, int flags = 0);
+  void                pause(Download* d, int flags = 0);
+
+  void                pause_default(Download* d) { pause(d); }
 
   void                check_hash(Download* d);
 
