@@ -46,8 +46,8 @@
 #include <rak/regex.h>
 #include <rak/path.h>
 #include <rak/string_manip.h>
-#include <sigc++/bind.h>
-#include <sigc++/hide.h>
+#include <sigc++/adaptors/bind.h>
+#include <sigc++/adaptors/hide.h>
 #include <torrent/object.h>
 #include <torrent/connection_manager.h>
 #include <torrent/error.h>
@@ -251,7 +251,7 @@ Manager::cleanup() {
 void
 Manager::shutdown(bool force) {
   if (!force)
-    std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::pause), m_downloadList));
+    std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::pause_default), m_downloadList));
   else
     std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::close_quick), m_downloadList));
 }
@@ -397,7 +397,7 @@ Manager::receive_http_failed(std::string msg) {
 void
 Manager::try_create_download(const std::string& uri, int flags, const command_list_type& commands) {
   // Adding download.
-  DownloadFactory* f = new DownloadFactory(uri, this);
+  DownloadFactory* f = new DownloadFactory(this);
 
   f->variables()["tied_to_file"] = (int64_t)(bool)(flags & create_tied);
   f->commands().insert(f->commands().end(), commands.begin(), commands.end());
@@ -405,7 +405,12 @@ Manager::try_create_download(const std::string& uri, int flags, const command_li
   f->set_start(flags & create_start);
   f->set_print_log(!(flags & create_quiet));
   f->slot_finished(sigc::bind(sigc::ptr_fun(&rak::call_delete_func<core::DownloadFactory>), f));
-  f->load();
+
+  if (flags & create_raw_data)
+    f->load_raw_data(uri);
+  else
+    f->load(uri);
+
   f->commit();
 }
 
@@ -460,11 +465,16 @@ path_expand(std::vector<std::string>* paths, const std::string& pattern) {
 
 bool
 manager_equal_tied(const std::string& path, Download* download) {
-  return path == rpc::call_command_d_string("d.get_tied_to_file", download);
+  return path == rpc::call_command_string("d.get_tied_to_file", rpc::make_target(download));
 }
 
 void
 Manager::try_create_download_expand(const std::string& uri, int flags, command_list_type commands) {
+  if (flags & create_raw_data) {
+    try_create_download(uri, flags, commands);
+    return;
+  }
+
   std::vector<std::string> paths;
   paths.reserve(256);
 
@@ -504,7 +514,7 @@ Manager::receive_hashing_changed() {
       continue;
 
     bool tryQuick =
-      rpc::call_command_d_value("d.get_hashing", *itr) == Download::variable_hashing_initial &&
+      rpc::call_command_value("d.get_hashing", rpc::make_target(*itr)) == Download::variable_hashing_initial &&
       (*itr)->download()->file_list()->bitfield()->empty();
 
     if (!tryQuick && foundHashing)
@@ -527,7 +537,7 @@ Manager::receive_hashing_changed() {
         (*itr)->download()->hash_stop();
 
         if (foundHashing) {
-          rpc::call_command_d_set_value("d.set_hashing", *itr, Download::variable_hashing_rehash);
+          rpc::call_command_set_value("d.set_hashing", Download::variable_hashing_rehash, rpc::make_target(*itr));
           continue;
         }
       }
@@ -538,7 +548,7 @@ Manager::receive_hashing_changed() {
     } catch (torrent::local_error& e) {
       if (tryQuick) {
         // Make sure we don't repeat the quick hashing.
-        rpc::call_command_d_set_value("d.set_hashing", *itr, Download::variable_hashing_rehash);
+        rpc::call_command_set_value("d.set_hashing", Download::variable_hashing_rehash, rpc::make_target(*itr));
 
       } else {
         (*itr)->set_hash_failed(true);
