@@ -39,7 +39,7 @@
 #include <cstdlib>
 #include <iostream>
 #include <string>
-#include <sigc++/bind.h>
+#include <sigc++/adaptors/bind.h>
 #include <torrent/http.h>
 #include <torrent/torrent.h>
 #include <torrent/exceptions.h>
@@ -103,12 +103,12 @@ load_session_torrents(Control* c) {
   std::list<std::string> l = c->core()->download_store()->get_formated_entries().make_list();
 
   for (std::list<std::string>::iterator first = l.begin(), last = l.end(); first != last; ++first) {
-    core::DownloadFactory* f = new core::DownloadFactory(*first, c->core());
+    core::DownloadFactory* f = new core::DownloadFactory(c->core());
 
     // Replace with session torrent flag.
     f->set_session(true);
     f->slot_finished(sigc::bind(sigc::ptr_fun(&rak::call_delete_func<core::DownloadFactory>), f));
-    f->load();
+    f->load(*first);
     f->commit();
   }
 }
@@ -117,12 +117,12 @@ void
 load_arg_torrents(Control* c, char** first, char** last) {
   //std::for_each(begin, end, std::bind1st(std::mem_fun(&core::Manager::insert), &c->get_core()));
   for (; first != last; ++first) {
-    core::DownloadFactory* f = new core::DownloadFactory(*first, c->core());
+    core::DownloadFactory* f = new core::DownloadFactory(c->core());
 
     // Replace with session torrent flag.
     f->set_start(true);
     f->slot_finished(sigc::bind(sigc::ptr_fun(&rak::call_delete_func<core::DownloadFactory>), f));
-    f->load();
+    f->load(*first);
     f->commit();
   }
 }
@@ -165,7 +165,10 @@ main(int argc, char** argv) {
     // torrent::ConnectionManager* are valid etc.
     initialize_commands();
 
-    rpc::parse_command_multiple(
+    rpc::parse_command_multiple(rpc::make_target(),
+//       "set_name = $cat={$system.hostname=,:,$system.pid=}\n"
+       "set_name = \"$cat=$system.hostname=,:,$system.pid=\"\n"
+
        // Currently not doing any sorting on main.
        "view_add = main\n"
        "view_add = default\n"
@@ -211,6 +214,13 @@ main(int argc, char** argv) {
        "view_sort_new = seeding,state_changed\n"
        "view_sort_current = seeding,state_changed_reverse\n"
 
+       // Changing these will bork the (non-existant) scheduler.
+       "view_add = scheduler\n"
+       "view_sort_new = scheduler,state_changed\n" // add started?
+       "view_sort_current = scheduler,state_changed\n"
+
+       //    "schedule = scheduler,10,10,download_scheduler=\n"
+
        "schedule = view_main,10,10,\"view_sort=main,20\"\n"
        "schedule = view_name,10,10,\"view_sort=name,20\"\n"
        //     "schedule = view_started,10,10,view_sort=started,5\n"
@@ -218,22 +228,16 @@ main(int argc, char** argv) {
        //     "schedule = view_complete,10,10,view_sort=complete,5\n"
        //     "schedule = view_incomplete,10,10,view_sort=incomplete,5\n"
 
-       //"schedule = scheduler,10,10,download_scheduler=\n"
        "schedule = session_save,1800,1800,session_save=\n"
        "schedule = low_diskspace,5,60,close_low_diskspace=500M\n"
 
-       // Changing these will bork the (non-existant) scheduler.
-       "view_add = scheduler\n"
-       "view_sort_new = scheduler,state_changed\n" // add started?
-       "view_sort_current = scheduler,state_changed\n"
-
-       //    "schedule = scheduler,10,10,download_scheduler=\n"
+       "encryption=allow_incoming,prefer_plaintext,enable_retry\n"
     );
 
     if (OptionParser::has_flag('n', argc, argv))
       control->core()->push_log("Ignoring ~/.rtorrent.rc.");
     else
-      rpc::parse_command_single("try_import = ~/.rtorrent.rc");
+      rpc::parse_command_single(rpc::make_target(), "try_import = ~/.rtorrent.rc");
 
     int firstArg = parse_options(control, argc, argv);
 
