@@ -62,10 +62,14 @@ public:
   static const int dialect_i8      = 1;
   static const int dialect_apache  = 2;
 
+  // These need to match CommandMap type values.
   static const int call_generic    = 0;
-  static const int call_download   = 1;
-  static const int call_file       = 2;
-  static const int call_tracker    = 3;
+  static const int call_any        = 1;
+  static const int call_download   = 2;
+  static const int call_peer       = 3;
+  static const int call_tracker    = 4;
+  static const int call_file       = 5;
+  static const int call_file_itr   = 6;
 
   XmlRpc() : m_env(NULL), m_registry(NULL), m_dialect(dialect_i8) {}
 
@@ -76,7 +80,7 @@ public:
 
   bool                process(const char* inBuffer, uint32_t length, slot_write slotWrite);
 
-  void                insert_command(const char* name, const char* parm, const char* doc, int call);
+  void                insert_command(const char* name, const char* parm, const char* doc);
 
   int                 dialect() { return m_dialect; }
   void                set_dialect(int dialect);
